'' be sure to set parallax serial terminal to 19200
''TriPingRay ping rover by Jim Gilbert n8rhq@comcast.net (with left, forward, and right facing ping sensors)
''written for the Parallax MSR1 Stingray with Parallax 2x16 Serial LCD on pin 15
''and Parallax Ping)))s on pins 0,1,2
''leds are on pins 3,4,5 to indicate direction of travel

Con    

  _clkmode = xtal1 + pll16x
  _clkfreq = 80_000_000
    
  lpg       = 0                               ''left ping i/o           
  cpg       = 1                               ''center ping i/o        
  rpg       = 2                               ''right ping i/o          
  leftLED   = 3                               ''left (port) side LED       - red
  centerLED = 4                               ''center LED                 - blue
  rightLED  = 5                               ''right (starboard) side LED - green
  TX_PIN    = 15                              ''LCD data i/o
  pwm       = 24                              ''pwm base i/o (for stingray's wheel motors)
  BAUD      = 19_200                          ''speed for LCD communication
 
Obj
    
  pg    : "ping"                              ''ping sensor object
  pw    : "pwmx8"                             ''pulse width modulation object
  LCD   : "LCD_DisplayDriver.spin"            ''LCD display object
  Debug : "FullDuplexSerial"                  ''standard from propeller library    
Var

  byte lrange[10]                             ''array for left ping values
  byte crange[10]                             ''array for center ping values
  byte rrange[10]                             ''array for right ping values
  long Stack[15]                              ''stack pointer
  byte l                                      ''current value for left ping
  byte r                                      ''current value for right ping
  byte c                                      ''current value for center ping
  byte loop_id                                ''loop identifier
  
Pub main |  lp, rp, i                         ''lp and rp used to set motor speed, i is just a counter
  
  dira[leftLED..rightLED]~~                   ''set LED pins (pin 3,4,5) to outputs           
  outa[leftLED..rightLED]~                    ''set all LED's off                             
                                               
  cognew(pings, @Stack)                       ''start pings in a new cog
  
  LCD.Init(TX_PIN, BAUD, true, 2)             ''initialize LCD_DisplayDriver.spin
  waitcnt(clkfreq / 100 + cnt)                ''pause for initialize

  debug.start(31,30,0,19200)                    ''start debugger
  debug.str(string(16))                         ''blank the screen 
  debug.str(string("L   C    R  L#"))           ''display formatting 

  pw.start(pwm, %00001111, 23_000)            ''start pwm object on base i/o for stingray wheel motors drive pins @ 23khz 
  loop_id := 0  
  LCD.ClearDisplay                            ''clear the display
  LCD.BacklightOn                             ''turn on the display backlight
  LCD.OutText(string("L    C   R"))           ''display formatting
  
 
''main loop
repeat                                        ''keep repeating instructions below
  display                                     ''display l, c, and r
                                              ''distances being returned for pings are in inches, therefore
                                              ''any compares done with l,r,c,lrange,rrange are assumed to be inches
  if c > 15                                   ''center distance greater than 10 inches, continue forward
    lp :=  150                                ''set left wheel speed
    rp := -150                                ''set right wheel speed (negative values means forward rotation)
    outa[leftLED..rightLED] := %010           ''turn on center (blue) LED
    move_robot(lp,rp)                         ''call move function using left and right wheel speed values
    loop_id := 1
    display                                   ''display l, c, and r
      
  else                                        ''center distance less 10 inches, decide which way to turn        
    if l < r                                  ''left distance less than right distance, rotate right
      outa[leftLED..rightLED] := %001         ''turn on right(starboard-green) side LED
      repeat until c > 12 or l > 10          ''spin cycle
        lp := 125                             ''set left wheel speed
        rp := 125                             ''set right wheel speed (positive value means reverse rotation)
        move_robot(lp,rp)                     ''call move function using left and right wheel speed values
        loop_id := 2
        display                               ''display l, c, and r
          
    else                                      ''right distance less than left distance, rotate left
      outa[leftLED..rightLED] := %100         ''turn on left(port-red) side LED
      repeat until c > 12 or r > 10          ''spin cycle  
        lp := -125                            ''set left wheel speed (negative value means reverse rotation)
        rp := -125                            ''set right wheel speed (negative value means forward rotation)
        move_robot(lp,rp)                     ''call move function using left and right wheel speed values
        loop_id := 3
        display                               ''display l, c, and r

  if l < 8                                    ''too close on left side?
    repeat until l > 10                       ''veer right
      lp := 125                               ''set left wheel speed
      rp := 125                               ''set right wheel speed (negative values means forward rotation)
      outa[leftLED..rightLED] := %001         ''turn on right(starboard-green) side LED
      move_robot(lp,rp)                       ''call move function using left and right wheel speed values
      loop_id := 4
      display                                 ''display l, c, and r

  if r < 8                                    ''too close on right side?
    repeat until r > 10                       ''veer left
      lp := -125                              ''set left wheel speed
      rp := -125                              ''set right wheel speed (negative values means forward rotation)
      outa[leftLED..rightLED] := %100         ''turn on left(port-red) side LED
      move_robot(lp,rp)                       ''call move function using left and right wheel speed values
      loop_id := 5
      display                                 ''display l, c, and r

  if lrange[0] == lrange[9] or rrange[0] == rrange[9] ''stuck??
    if l < r                                  ''left distance less than right distance, rotate right
      outa[leftLED..rightLED] := %111         ''turn on all LED's - all on for reverse
      repeat i from 0 to 5                    ''let's try reversing for a while   
         lp := -125                           ''set left wheel speed (negative value means reverse rotation)
         rp :=  125                           ''set right wheel speed (positive value means reverse rotation)
         move_robot(lp,rp)                    ''call move function using left and right wheel speed values
         loop_id := 6
         display                              ''display l, c, and r
      repeat until c > 12 or l > 10          ''spin cycle, l < r, so spin right until l > 10 and c > 12  
        lp := 125                             ''set left wheel speed
        rp := 125                             ''set right wheel speed (positive value means reverse rotation)
        move_robot(lp,rp)                     ''call move function using left and right wheel speed values
        loop_id := 7
        display                               ''display l, c, and r
    else                                          ''right distance less than left distance, rotate left
      outa[leftLED..rightLED] := %111         ''turn on all LED's - all on for reverse
      repeat i from 0 to 5                    ''let's try reversing for a while   
         lp := -125                           ''set left wheel speed (negative value means reverse rotation)
         rp :=  125                           ''set right wheel speed (positive value means reverse rotation)
         move_robot(lp,rp)                    ''call move function using left and right wheel speed values
         loop_id := 8
         display                              ''display l, c, and r
      repeat until c > 12 or r > 10          ''spin cycle, l > r, so spin left until r > 10 and c > 12  
        lp := -125                            ''set left wheel speed (negative value means reverse rotation)
        rp := -125                            ''set right wheel speed (negative value means forward rotation)
        move_robot(lp,rp)                     ''call move function using left and right wheel speed values
        loop_id := 9
        display                               ''display l, c, and r
         
Pub move_robot(left_wheel, right_wheel)       ''set the speed of stingray's left and right wheels,                
                                              ''-255 max.reverse to 255 max.forward, 0=stop
  left_wheel := -255 #> left_wheel <# 255     ''limit left_wheel motor speed from -255 to 255                                                                                                                                       
  right_wheel := -255 #> right_wheel <# 255   ''limit right_wheel motor speed from -255 to 255

  ifnot left_wheel & $80_00_00_00             ''check sign bit for left_wheel motor speed amount 
    pw.duty(pwm + 1, left_wheel & $ff)        ''set pwm output for left_wheel base pin + 1 limited to 255
    pw.duty(pwm, 0)                           ''set pwm output for pwm base pin to 0 
  else                                        
    pw.duty(pwm + 1, 0)                       ''set pwm output for pwm base pin + 1 to 0
    pw.duty(pwm, (0 - left_wheel) & $ff)      ''set pwm output for left_wheel base pin limited to 255
     
  ifnot right_wheel & $80_00_00_00            ''check sign bit for right_wheel motor speed amount 
    pw.duty(pwm + 2, right_wheel & $ff)       ''set pwm output for right_wheel base pin + 2 limited to 255
    pw.duty(pwm + 3, 0)                       ''set pwm output for right_wheel base pin + 3 to 0
  else                                        
    pw.duty(pwm + 2, 0)                       ''set pwm output for right_wheel base pin + 2 to 0
    pw.duty(pwm + 3, (0 - right_wheel) & $ff) ''set pwm output for right_wheel base pin + 3 limited to 255 

PUB pings | i                                 '' runs in a seperate cog   i is just a counter
                                              ''continuously read in PING))) distances and store in an independant array
 repeat
  
   repeat i from 0 to 9
  
     lrange[i] := pg.Inches(lpg)
     l := lrange[i]
     waitcnt(clkfreq/100 + cnt)               ''delay 10ms before next ping distance check (echo reduction)

     crange[i] := pg.Inches(cpg)
     c := crange[i]
     waitcnt(clkfreq/100 + cnt)               ''delay 10ms before next ping distance check (echo reduction)

     rrange[i] := pg.Inches(rpg)
     r := rrange[i]
     waitcnt(clkfreq/100 + cnt)               ''delay 10ms before next ping distance check (echo reduction)
     
PUB display                                   ''displays value of l, c, r on LCD

   debug.dec(l)
   debug.str(string(" "))
   debug.dec(c)
   debug.str(string(" "))
   debug.dec(r)
   debug.str(string(" "))
   debug.dec(loop_id)
   debug.str(string(13))
   
   LCD.GotoRowCol(0, 15)                      ''position cursor on 1st row, last column
   LCD.OutNum(loop_id)                        ''display current value for loop_id

   LCD.GotoRowCol(1, 0)                       ''position cursor on 2nd row, 1st column
   LCD.OutNum(l)                              ''display current value for l
   LCD.OutText(string("   "))                 ''display formatting
   LCD.OutNum(c)                              ''display current value for c
   LCD.OutText(string("   "))                 ''display formatting
   LCD.OutNum(r)                              ''display current value for r
   LCD.OutText(string("  "))                  ''display formatting
    